#ifndef OPENSSL_HEADER_CHACHA_H
#define OPENSSL_HEADER_CHACHA_H

#ifdef  __cplusplus
extern "C" {
#endif

#define CHACHA_KEY_LEN 32

typedef struct chacha_ctx_s {
	unsigned char state[48];
	size_t rounds;
	unsigned char buffer[64];
	size_t leftover;
} CHACHA_CTX;

int CHACHA_test_next_opt(void);

size_t CHACHA_get_parallel_blocks(void);
void CHACHA_init(CHACHA_CTX *ctx, const unsigned char *key, const unsigned char *iv, size_t ivlen, size_t rounds);
void CHACHA_init_iv(CHACHA_CTX *ctx, const unsigned char *iv, size_t ivlen);
void CHACHA_update(CHACHA_CTX *ctx, const unsigned char *in, size_t inl, unsigned char *out, size_t *outl);
void CHACHA_update_and_finish(CHACHA_CTX *ctx, const unsigned char *in, size_t inl, unsigned char *out, size_t *outl);
void CHACHA_finish(CHACHA_CTX *ctx, unsigned char *out, size_t *outl);

size_t CHACHA20_get_parallel_blocks(void);
void CHACHA20_init(CHACHA_CTX *ctx, const unsigned char *key, const unsigned char *iv, size_t ivlen);
void CHACHA20_init_iv(CHACHA_CTX *ctx, const unsigned char *iv, size_t ivlen);
void CHACHA20_update(CHACHA_CTX *ctx, const unsigned char *in, size_t inl, unsigned char *out, size_t *outl);
void CHACHA20_update_and_finish(CHACHA_CTX *ctx, const unsigned char *in, size_t inl, unsigned char *out, size_t *outl);
void CHACHA20_finish(CHACHA_CTX *ctx, unsigned char *out, size_t *outl);

#if defined(__cplusplus)
}  /* extern C */
#endif

#endif /* OPENSSL_HEADER_CHACHA_H */

