#ifndef OPENSSL_HEADER_POLY1305_H
#define OPENSSL_HEADER_POLY1305_H

#ifdef  __cplusplus
extern "C" {
#endif

#define POLY1305_TAG_LEN 16
#define POLY1305_KEY_LEN 32

typedef struct poly1305_ctx {
	unsigned char state[192]; /* largest state (AVX2) */
	unsigned char buffer[64]; /* largest block size (AVX2) */
	size_t leftover;
	size_t block_size;
} POLY1305_CTX;

int POLY1305_test_next_opt(void);

size_t POLY1305_get_block_size(void);
void POLY1305_init(POLY1305_CTX *ctx, const unsigned char *key);
void POLY1305_init_ex(POLY1305_CTX *ctx, const unsigned char *key, size_t bytes_hint);
void POLY1305_update(POLY1305_CTX *ctx, const unsigned char *in, size_t inl);
void POLY1305_finish(POLY1305_CTX *ctx, unsigned char *mac);
void POLY1305(unsigned char *mac, const unsigned char *in, size_t inl, const unsigned char *key);

int POLY1305_verify(const unsigned char *mac1, const unsigned char *mac2);

#if defined(__cplusplus)
}  /* extern C */
#endif

#endif /* OPENSSL_HEADER_POLY1305_H */

